# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Class representing a single Python environment.

Includes support for:
  - fetching information about a specific Python environment
  - executing a command in a specific Python environment

"""

import sys
have_subprocess_devnull = sys.version_info >= (3, 3)
if not have_subprocess_devnull:
    import os
import subprocess


class BadEnvironment(Exception):
    """
    Problem occurred while scanning a Python environment.

    The problem may be either a technical one in the scanning process itself,
    or it may be a problem with something detected about the Python environment
    in question, e.g. it might be using an incompatible Python version.

    Specifying the environment scan result information when constructing this
    exception is optional and may be added to an exception later on using the
    set_raw_scan_results() method.

    """

    def __init__(self, message, out=None, err=None, exit_code=None):
        Exception.__init__(self, message)
        self.message = message
        self.out = out
        self.err = err
        self.exit_code = exit_code

    def raw_scan_results(self):
        return self.out, self.err, self.exit_code

    def set_raw_scan_results(self, out, err, exit_code):
        assert self.out is None
        assert self.err is None
        assert self.exit_code is None
        self.out = out
        self.err = err
        self.exit_code = exit_code


class _UndefinedParameter:
    """Internal class used to indicate undefined parameter values."""
    pass


class Environment:
    """
    Represents a single Python environment.

    Allows running commands using the environment's Python interpreter.
    Allows running an initial Python environment scan to collect information
    about it.

    Note that most of the information about the environment will not be
    available until the initial environment scan is run.

    """

    def __init__(self, name, command):
        self.__name = name
        self.__command = command
        self.initial_scan_completed = False

    def command(self):
        return self.__command

    def description(self):
        return "%s on %s" % (self.sys_version, self.sys_platform)

    def execute(self, args=[], input=None, capture_output=False,
            cwd=_UndefinedParameter):
        stdin = subprocess.PIPE
        close_stdin = False
        if input is None:
            if have_subprocess_devnull:
                stdin = subprocess.DEVNULL
            else:
                stdin = open(os.devnull, "r")
                close_stdin = True
        try:
            kwargs = {}
            kwargs["stdin"] = stdin
            if capture_output:
                kwargs["stdout"] = subprocess.PIPE
                kwargs["stderr"] = subprocess.PIPE
            if cwd is not _UndefinedParameter:
                kwargs["cwd"] = cwd
            kwargs["universal_newlines"] = True
            popen = subprocess.Popen([self.command()] + args, **kwargs)
            out, err = popen.communicate(input)
        finally:
            if close_stdin:
                stdin.close()
        return out, err, popen.returncode

    def name(self):
        return self.__name

    def run_initial_scan(self):
        self.initial_scan_completed = False
        scanner = self.__initial_scanner()
        try:
            scan_results = scanner.scan(self)
            self.__collect_scanned_values(scan_results)
            self.__parse_scanned_version_info()
            self.initial_scan_completed = True
            self.__check_python_version()
        except BadEnvironment:
            sys.exc_info()[1].set_raw_scan_results(*scanner.raw_scan_results())
            raise
        return scanner.raw_scan_results()

    def __check_python_version(self):
        if self.sys_version_info < (2, 4):
            raise BadEnvironment("Unsupported Python version (%s, %s-bit)"
                % (self.python_version, self.pointer_size_in_bits))

    def __collect_scanned_values(self, values):
        v = values
        # System.
        self.platform_architecture = v["platform.architecture"]
        self.pointer_size_in_bits = v["pointer size in bits"]
        self.sys_path = v["sys.path"]
        self.sys_version = v["sys.version"]
        self.sys_version_info_formatted = v["sys.version_info (formatted)"]
        self.sys_version_info_raw = v["sys.version_info (raw)"]
        self.sys_platform = v["sys.platform"]
        self.sys_executable = v["sys.executable"]
        # Packages.
        self.ctypes_version = v["ctypes version"]
        self.pip_version = v["pip version"]
        self.pytest_version = v["pytest version"]
        self.setuptools_version = v["setuptools version"]
        self.virtualenv_version = v["virtualenv version"]
        # Extra package info.
        self.setuptools_zipped_egg = v["setuptools zipped egg"]

    def __construct_python_version(self):
        """
        Construct a setuptools compatible Python version string.

        Constructed based on the environment's reported sys.version_info.

        """
        major, minor, micro, release_level, serial = self.sys_version_info
        assert release_level in ("alfa", "beta", "candidate", "final")
        assert release_level != "final" or serial == 0
        parts = [str(major), ".", str(minor), ".", str(micro)]
        if release_level != "final":
            parts.append(release_level[0])
            parts.append(str(serial))
        self.python_version = "".join(parts)

    @staticmethod
    def __initial_scanner():
        s = EnvironmentScanner()

        s.add_import("os.path")
        s.add_import("platform")
        s.add_import("struct")
        s.add_import("sys")

        s.add_function("""\
def version_info_string():
    major, minor, micro, release_level, serial = sys.version_info
    if not (isinstance(major, int) and isinstance(minor, int) and
        isinstance(micro, int) and isinstance(release_level, str) and
        isinstance(serial, int)) or "," in release_level:
        return ""
    # At least Python versions 2.7.6 & 3.1.3 require an explicit tuple() cast.
    return "%d,%d,%d,%s,%d" % tuple(sys.version_info)
""")
        s.add_function("""\
def setuptools_zipped_egg():
    try:
        from pkg_resources import (
            DistributionNotFound, EGG_DIST, get_distribution)
    except ImportError:
        return Skip
    try:
        d = get_distribution("setuptools")
    except DistributionNotFound:
        return Skip
    if d.precedence != EGG_DIST or not os.path.isfile(d.location):
        return Skip  # file = zipped egg; folder = unzipped egg
    return d.location
""")

        s.add_field("platform.architecture", "platform.architecture()")
        s.add_field("pointer size in bits", '8 * struct.calcsize("P")')
        s.add_field("sys.executable")
        s.add_field("sys.path")
        s.add_field("sys.platform")
        s.add_field("sys.version")
        s.add_field("sys.version_info (formatted)", "version_info_string()")
        s.add_field("sys.version_info (raw)", "sys.version_info")

        s.add_package_version_field("ctypes", default=None)
        s.add_package_version_field("pip", default=None)
        s.add_package_version_field("pytest", default=None)
        s.add_package_version_field("setuptools", default=None)
        s.add_package_version_field("virtualenv", default=None)

        s.add_field("setuptools zipped egg", "setuptools_zipped_egg()", None)
        return s

    def __parse_scanned_version_info(self):
        """Parses the environment's formatted version info string."""
        string = self.sys_version_info_formatted
        try:
            major, minor, micro, release_level, serial = string.split(",")
            if (release_level in ("alfa", "beta", "candidate", "final") and
                    (release_level != "final" or serial == "0") and
                    major.isdigit() and  # --- --- --- --- --- --- --- --- ---
                    minor.isdigit() and  # Explicit isdigit() checks to detect
                    micro.isdigit() and  # leading/trailing whitespace.
                    serial.isdigit()):   # --- --- --- --- --- --- --- --- ---
                self.sys_version_info = (int(major), int(minor), int(micro),
                    release_level, int(serial))
                self.__construct_python_version()
                return
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            pass
        raise BadEnvironment("Unsupported Python version (%s)" % (string,))


class EnvironmentScanner:
    """
    Allows scanning a given Python environment for specific information.

    Runs a scanner script in the given Python environment, crafted based on the
    information we wish to find out about the environment, and then collects
    and returns the requested information.

    """

    # Explicitly specified scan output start & finish markers allow us to
    # safely ignore extra output that might be added by the environment's
    # Python interpreter startup. This can be useful, for instance, if you want
    # to debug this script by having the Python interpreter startup script
    # output the exact calls made to the Python interpreter.
    SCAN_START_MARKER = "--- SCAN START ---"
    SCAN_FINISH_MARKER = "--- SCAN FINISH ---"

    __scanner_script__startup = """\
class Skip:
    pass

def print_field(id, value):
    if value is not Skip:
        print("%s: %s" % (id, value))
"""

    # If available, setuptools can give us more detailed version information
    # read from the package's meta-data than simply reading it from its
    # __version__ attribute. For example, in case of setuptools it can tell us
    # that we are dealing with version '3.7dev' while __version__ would tell us
    # just '3.7'. Also, some older packages, such as pip 1.1, may not have a
    # __version__ attribute set in their main module at all.
    __scanner_script__package_version_scanner = """\
def package_version(package_name):
    try:
        package = __import__(package_name, {}, {}, ("__version__",))
    except (KeyboardInterrupt, SystemExit):
        raise
    except Exception:
        return Skip  # No package - no version.
    try:
        version = package.__version__
    except AttributeError:
        version = "unknown"
    try:
        from pkg_resources import (DistributionNotFound, get_distribution)
    except ImportError:
        return version
    try:
        return get_distribution(package_name).version
    except DistributionNotFound:
        return version
"""

    def __init__(self):
        self.__out = None
        self.__err = None
        self.__exit_code = None
        self.__scan_for_version_info = False
        self.__extra_script_functions = []
        self.__extra_script_imports = []
        self.__fields = []

    def add_field(self, id, getter=None, default=_UndefinedParameter):
        if getter is None:
            getter = id
        self.__add_field(id, "print_field(%r, %s)" % (id, getter), default)

    def add_function(self, code):
        self.__extra_script_functions.append(code)

    def add_import(self, module_name):
        self.__extra_script_imports.append(module_name)

    def add_package_version_field(self, package_name, default=_UndefinedParameter):
        self.__scan_for_version_info = True
        field_id = "%s version" % (package_name,)
        field_getter = "print_field(%r, package_version(%r))" % (
            field_id, package_name)
        self.__add_field(field_id, field_getter, default)

    def raw_scan_results(self):
        return self.__out, self.__err, self.__exit_code

    def scan(self, environment):
        self.__scan(environment)
        raw_data = self.__parse_raw_scanner_output()
        return self.__map_raw_data_to_expected_fields(raw_data)

    def __add_field(self, id, getter, default):
        assert id not in [x[0] for x in self.__fields]
        self.__fields.append((id, getter, default))

    @staticmethod
    def __extract_value(raw_data, id, default):
        if default is not _UndefinedParameter:
            return raw_data.pop(id, default)
        try:
            return raw_data.pop(id)
        except KeyError:
            raise BadEnvironment("Missing scan output record (%s)" % (
                sys.exc_info()[1],))

    def __map_raw_data_to_expected_fields(self, raw_data):
        scanner_results = {}
        for id, getter, default in self.__fields:
            assert id not in scanner_results
            scanner_results[id] = self.__extract_value(raw_data, id, default)
        if raw_data:
            raise BadEnvironment("Extra scan output records (%s)" % (
                ",".join(raw_data.keys()),))
        return scanner_results

    def __parse_raw_scanner_output(self):
        assert self.__scanned()
        result = {}
        in_scanner_output = False
        for line in self.__out.split("\n"):
            if not in_scanner_output:
                in_scanner_output = line.startswith(self.SCAN_START_MARKER)
                continue
            if line.startswith(self.SCAN_FINISH_MARKER):
                return result
            split_result = line.split(": ", 1)
            if len(split_result) != 2:
                raise BadEnvironment("Error parsing scan output record")
            key, value = split_result
            if key in result:
                raise BadEnvironment("Duplicate scan output record (%s)" %
                    (key,))
            result[key] = value
        if not in_scanner_output:
            raise BadEnvironment("No valid scan output detected")
        raise BadEnvironment("Scan output truncated")

    def __scan(self, environment):
        assert not self.__scanned()
        try:
            self.__out, self.__err, self.__exit_code = environment.execute(
                input=self.__scanner_script(), capture_output=True)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            e_type, e = sys.exc_info()[:2]
            try:
                raise BadEnvironment("%s: %s" % (e_type.__name__, e))
            finally:
                del e  # explicitly break circular reference chain in Python 3
        if self.__exit_code != 0:
            raise BadEnvironment("Scan failed (exit code %d)" %
                (self.__exit_code,))
        if self.__err:
            raise BadEnvironment("Scan failed (error output detected)")
        if not self.__out:
            raise BadEnvironment("Scan failed (no output)")

    def __scanned(self):
        return self.__exit_code is not None

    def __scanner_script(self):
        script_lines = []
        if self.__extra_script_imports:
            for x in self.__extra_script_imports:
                script_lines.append("import %s" % (x,))
            script_lines.append("")
        script_lines.append(self.__scanner_script__startup)
        script_lines.append("")
        if self.__scan_for_version_info:
            script_lines.append(self.__scanner_script__package_version_scanner)
        script_lines.extend(self.__extra_script_functions)
        script_lines.append("")
        script_lines.append('print("%s")' % (self.SCAN_START_MARKER,))
        script_lines.extend(getter for id, getter, default in self.__fields)
        script_lines.append('print("%s")' % (self.SCAN_FINISH_MARKER,))
        script_lines.append("")
        return "\n".join(script_lines)



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Dependency sort unit tests.

Implemented using the 'pytest' testing framework.

"""

import testutils
if __name__ == "__main__":
    testutils.run_using_pytest(globals())

from suds.xsd.depsort import dependency_sort

import pytest
from six import iteritems

import copy


# some of the tests in this module make sense only with assertions enabled
# (note though that pytest's assertion rewriting technique, as used by default
# in recent pytest releases, will keep assertions enabled inside the used test
# modules even when the underlying Python interpreter has been run using the -O
# command-line option)
try:
    assert False
except AssertionError:
    assertions_enabled = True
else:
    assertions_enabled = False


# shared test data

# f --+-----+-----+
# |   |     |     |
# |   v     v     v
# |   e --> d --> c --> b
# |   |           |     |
# +---+-----------+-----+--> a --> x
_test_dependency_tree = {
    "x": (),
    "a": ("x",),
    "b": ("a",),
    "c": ("a", "b"),
    "d": ("c",),
    "e": ("d", "a"),
    "f": ("e", "c", "d", "a")}


def test_dependency_sort():
    dependency_tree = _test_dependency_tree
    result = dependency_sort(dependency_tree)
    assert sorted(result) == sorted(iteritems(dependency_tree))
    _assert_dependency_order((x[0] for x in result), dependency_tree)


def test_dependency_sort_does_not_mutate_input():
    dependency_tree = _test_dependency_tree

    # save the original dependency tree structure information
    expected_deps = {}
    expected_deps_ids = {}
    for x, y in iteritems(dependency_tree):
        expected_deps[x] = copy.copy(y)
        expected_deps_ids[id(x)] = id(y)

    # run the dependency sort
    dependency_sort(dependency_tree)

    # verify that the dependency tree structure is unchanged
    assert len(dependency_tree) == len(expected_deps)
    for key, deps in iteritems(dependency_tree):
        # same deps for each key
        assert id(deps) == expected_deps_ids[id(key)]
        # deps structure compare with the original copy
        assert deps == expected_deps[key]
        # explicit deps content id matching just in case the container's __eq__
        # is not precise enough
        _assert_same_content_set(deps, expected_deps[key])


###############################################################################
#
# Test utilities.
#
###############################################################################

def _assert_dependency_order(sequence, dependencies):
    """
    Assert that a sequence is ordered dependencies first.

    The only way an earlier entry is allowed to have a later entry as its
    dependency is if they are both part of the same dependency cycle.

    """
    sequence = list(sequence)
    dependency_closure = _transitive_dependency_closure(dependencies)
    for i, a in enumerate(sequence):
        for b in sequence[i + 1:]:
            a_dependent_on_b = b in dependency_closure[a]
            b_dependent_on_a = a in dependency_closure[b]
            assert b_dependent_on_a or not a_dependent_on_b


def _assert_same_content_set(lhs, rhs):
    """Assert that two iterables have the same content (order independent)."""
    counter_lhs = _counter(lhs)
    counter_rhs = _counter(rhs)
    assert counter_lhs == counter_rhs


def _counter(iterable):
    """Return an {id: count} dictionary for all items from `iterable`."""
    counter = {}
    for x in iterable:
        counter[id(x)] = counter.setdefault(id(x), 0) + 1
    return counter


def _transitive_dependency_closure(dependencies):
    """
    Returns a transitive dependency closure.

    If target A is dependent on target B, and target B is in turn dependent on
    target C, then target A is also implicitly dependent on target C. A
    transitive dependency closure is an expanded dependency collection so that
    in it all such implicit dependencies have been explicitly specified.

    """
    def clone(deps):
        return dict((k, set(v)) for k, v in iteritems(deps))
    closure = None
    new = clone(dependencies)
    while new != closure:
        closure = clone(new)
        for k, deps in iteritems(closure):
            for dep in deps:
                new[k] |= closure[dep]
    return closure


###############################################################################
#
# Test utility tests.
#
###############################################################################

@pytest.mark.skipif(not assertions_enabled, reason="assertions disabled")
@pytest.mark.parametrize("sequence, dependencies", (
    (["x", "y"], {"x": ("y",), "y": ()}),
    (["x", "y"], {"x": ("z",), "z": ("y",), "y": ()}),
    (["y", "x", "z"], {"x": ("z",), "z": ("y",), "y": ()}),
    (["z", "y", "x"], {"x": ("z",), "z": ("y",), "y": ()}),
    # unrelated element groups
    (["y", "a", "x", "b"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["x", "b", "y", "a"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["b", "x", "y", "a"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["a", "y", "b", "x"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["a", "b", "y", "x"], {"x": (), "y": ("x",), "a": (), "b": ("a",)})))
def test_assert_dependency_order__invalid(sequence, dependencies):
    pytest.raises(AssertionError, _assert_dependency_order, sequence,
        dependencies)


@pytest.mark.parametrize("sequence, dependencies", (
    (["y", "x"], {"x": ("y",), "y": ()}),
    (["y", "x"], {"x": ("z",), "z": ("y",), "y": ()}),
    (["y", "z", "x"], {"x": ("z",), "z": ("y",), "y": ()}),
    # unrelated element groups
    (["x", "y", "a", "b"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["x", "a", "y", "b"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["a", "x", "y", "b"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["a", "x", "b", "y"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    (["a", "b", "x", "y"], {"x": (), "y": ("x",), "a": (), "b": ("a",)}),
    # dependency cycle
    (["x", "y"], {"x": ("y",), "y": ("x",)}),
    (["y", "x"], {"x": ("y",), "y": ("x",)}),
    # elements not mentioned in the dependency tree
    ([1], {}),
    (["a"], {"x": ("y",), "y": ()})))
def test_assert_dependency_order__valid(sequence, dependencies):
    _assert_dependency_order(sequence, dependencies)


@pytest.mark.skipif(not assertions_enabled, reason="assertions disabled")
@pytest.mark.parametrize("lhs, rhs", (
    # empty
    #    ([1, 2.0, 6], [1, 2, 6]),
    ((), (1,)),
    ([2], []),
    ([], (4, 2)),
    ([], (x for x in [8, 4])),
    ((x for x in [1, 1]), []),
    # without duplicates
    ([1, 2, 3], [1, 2, 4]),
    ([1, 2, 3], [1, 2]),
    ([1, 2, 3], [1, 4]),
    ([0], [0.0]),
    ([0], [0.0]),
    # with duplicates
    ([1, 1], [1]),
    ((x for x in [1, 1]), [1]),
    ([1, 1], [1, 2, 1]),
    ([1, 1, 2, 2], [1, 2, 1]),
    # different object ids
    ([object()], [object()])))
def test_assert_same_content_set__invalid(lhs, rhs):
    pytest.raises(AssertionError, _assert_same_content_set, lhs, rhs)


@pytest.mark.parametrize("lhs, rhs", (
    # empty
    ((), ()),
    ([], []),
    ([], ()),
    ([], (x for x in [])),
    ((x for x in []), []),
    # matching without duplicates
    ([1, 2, 6], [1, 2, 6]),
    ([1, 2, 6], [6, 2, 1]),
    # matching with duplicates
    ([1, 2, 2, 6], [6, 2, 1, 2]),
    # matching object ids
    ([_assert_same_content_set], [_assert_same_content_set])))
def test_assert_same_content_set__valid(lhs, rhs):
    _assert_same_content_set(lhs, rhs)


def test_counter():
    a = object()
    b = object()
    c = object()
    d = object()
    input = [a, b, b, c, c, d, a, a, a, d, b, b, b, b, b, a, d]
    result = _counter(input)
    assert len(result) == 4
    assert result[id(a)] == input.count(a)
    assert result[id(b)] == input.count(b)
    assert result[id(c)] == input.count(c)
    assert result[id(d)] == input.count(d)


def test_counter__empty():
    assert _counter([]) == {}



# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jeff Ortel ( jortel@redhat.com )

import sys
sys.path.append('../../')

from suds import WebFault
from suds.client import Client

import traceback as tb


errors = 0


def start(url):
    print('\n______________________________________________________________\n')
    print('Test @ ( %s )' % (url,))

try:
    url = 'http://localhost:9090/jasperserver-pro/services/repository?wsdl'
    start(url)
    client = Client(url, username='jeff', password='ortel')
    print(client)
    print(client.service.list(''))
except WebFault, f:
    errors += 1
    print(f)
    print(f.fault)
except (KeyboardInterrupt, SystemExit):
    raise
except Exception:
    errors += 1
    print(sys.exc_info()[1])
    tb.print_exc()

print('\nFinished: errors = %d' % (errors,))



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Suds SAX module's special character encoder profiler.

"""

import suds.sax.enc
import tests.profiling

import sys


"""Profiler parameter constants."""
NONE = 0
SOME = 1
MANY = 2
QUANTITY = {NONE: "NONE", SOME: "SOME", MANY: "MANY"}


class Profiler(tests.profiling.ProfilerBase):

    def __init__(self, long_input, replacements, cdata,
            show_each_timing=False, show_minimum=True):
        assert long_input in (True, False)
        assert replacements in QUANTITY
        assert cdata in QUANTITY

        super(Profiler, self).__init__(show_each_timing, show_minimum)

        self.encode_input = self.__construct_input(long_input, replacements,
            cdata, encoded=False)
        self.decode_input = self.__construct_input(long_input, replacements,
            cdata, encoded=True)

        print("long_input=%s; replacements=%s; cdata=%s" % (long_input,
            QUANTITY[replacements], QUANTITY[cdata]))
        print("  encode input data length: %d" % (len(self.encode_input),))
        print("  decode input data length: %d" % (len(self.decode_input),))

    def decode(self):
        suds.sax.enc.Encoder().decode(self.decode_input)

    def encode(self):
        suds.sax.enc.Encoder().encode(self.encode_input)

    def __construct_input(self, long_input, replacements, cdata, encoded):
        """Construct profiling input data matching the given parameters."""
        basic_input = "All that glitters is not gold."
        replacements_input = "<>&'"
        if encoded:
            replacements_input = "&gt;&lt;&amp;&apos"
        cdata_input = "<![CDATA[yaba-daba-duba]]>"

        # Approximate constructed input data size in bytes.
        size = 500
        if long_input:
            size = 1000000

        border_input = basic_input
        if replacements != NONE:
            border_input += replacements_input
        if cdata != NONE:
            border_input += cdata_input
        if replacements == MANY:
            basic_input += replacements_input
        if cdata == MANY:
            basic_input += cdata_input

        border_size = 2 * len(border_input)
        middle_size = max(size - border_size, 0)
        middle_count = middle_size // len(basic_input) + 1
        input = [border_input, basic_input * middle_count, border_input]
        return "".join(input)


if __name__ == "__main__":
    print("Python %s" % (sys.version,))
    print("")
    p = Profiler(long_input=True, replacements=MANY, cdata=MANY)
    p.timeit('encode', 18)
    p.timeit('decode', 170)
    print("")
    p = Profiler(long_input=False, replacements=MANY, cdata=MANY)
    p.timeit('encode', 24000)
    p.timeit('decode', 140000)
    print("")
    p = Profiler(long_input=True, replacements=MANY, cdata=NONE)
    p.timeit('encode', 16)
    p.timeit('decode', 150)
    print("")
    p = Profiler(long_input=False, replacements=MANY, cdata=NONE)
    p.timeit('encode', 21000)
    p.timeit('decode', 140000)
    print("")
    p = Profiler(long_input=True, replacements=NONE, cdata=NONE)
    p.timeit('encode', 250)
    p.timeit('decode', 450)
    print("")
    p = Profiler(long_input=False, replacements=NONE, cdata=NONE)
    p.timeit('encode', 240000)
    p.timeit('decode', 200000)
    print("")
    p = Profiler(long_input=True, replacements=NONE, cdata=MANY)
    p.timeit('encode', 45)
    p.timeit('decode', 400)
    print("")
    p = Profiler(long_input=False, replacements=NONE, cdata=MANY)
    p.timeit('encode', 35000)
    p.timeit('decode', 190000)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Main installation and project management script for this project.

Attempts to use setuptools if available, and even attempts to install it
automatically if it is not, downloading it from PyPI if needed. However, its
main functionality will function just fine without setuptools as well. Having
setuptools available provides us with the following benefits:
  - setup.py 'egg_info' command constructing the project's metadata
  - setup.py 'develop' command deploying the project in 'development mode',
    thus making it available on sys.path, yet still editable directly in its
    source checkout - equivalent to running a pip based installation using
    'easy_install -e' or 'pip install -e'
  - setup.py 'test' command as a standard way to run the project's test suite
  - when running the installation directly from the source tree setup.py
    'install' command:
      - automatically installs the project's metadata so package management
        tools like pip recognize the installation correctly, e.g. this allows
        using 'pip uninstall' to undo the installation
      - package installed as a zipped egg by default

"""

import sys
if sys.version_info < (2, 4):
    print("ERROR: Python 2.4+ required")
    sys.exit(-2)
if (3,) <= sys.version_info < (3, 1):
    print("ERROR: Python 3.0 not supported - please use Python 3.1+ instead")
    sys.exit(-2)

import os
import os.path
import re

# Workaround for a Python issue detected with Python 3.1.3 when running our
# pytest based 'setup.py test' command. At the end of the test run, Python
# would report an error:
#
#   > Error in atexit._run_exitfuncs:
#   > TypeError: print_exception(): Exception expected for value, str found
#
# Workaround found suggested by Richard Oudkerk in Python's issue tracker at:
#   http://bugs.python.org/issue15881#msg170215
#
# The error is caused by two chained Python bugs:
#  1. The multiprocessing module seems to call its global cleanup function only
#     after all of its globals have already been released, thus raising an
#     exception.
#  2. atexit exception handling implementation is not prepared to handle and
#     correctly report the exception as raised by the multiprocessing module
#     cleanup.
if (3,) <= sys.version_info < (3, 2):
    import multiprocessing
    del multiprocessing


# -----------------------------------------------------------------------------
# Global variables.
# -----------------------------------------------------------------------------

distutils_cmdclass = {}
extra_setup_params = {}

# Hardcoded configuration.
attempt_to_use_setuptools = True
attempt_to_install_setuptools = True


# -----------------------------------------------------------------------------
# Detect the setup.py environment - current & script folder.
# -----------------------------------------------------------------------------

# Setup documentation incorrectly states that it will search for packages
# relative to the setup script folder by default when in fact it will search
# for them relative to the current working folder. It seems avoiding this
# problem cleanly and making the setup script runnable with any current working
# folder would require better distutils and/or setuptools support.
# Attempted alternatives:
#   * Changing the current working folder internally makes any passed path
#     parameters be interpreted relative to the setup script folder when they
#     should be interpreted relative to the initial current working folder.
#   * Passing the script folder to setup() using the parameter
#       package_dir={"": script_folder}
#     makes the setup 'build' command work from any folder but 'egg-info'
#     (together with any related commands) still fails.
script_folder = os.path.realpath(os.path.dirname(__file__))
current_folder = os.path.realpath(os.getcwd())
if script_folder != current_folder:
    print("ERROR: Suds library setup script needs to be run from the folder "
        "containing it.")
    print("")
    print("Current folder: %s" % current_folder)
    print("Script folder: %s" % script_folder)
    sys.exit(-2)


# -----------------------------------------------------------------------------
# Import suds_devel module shared between setup & development scripts.
# -----------------------------------------------------------------------------

tools_folder = os.path.join(script_folder, "tools")
sys.path.insert(0, tools_folder)
import suds_devel
sys.path.pop(0)

from suds_devel.requirements import (check_Python24_pytest_requirements,
    pytest_requirements, six_requirements)


# -----------------------------------------------------------------------------
# Attempt to use setuptools for this installation.
# -----------------------------------------------------------------------------
# setuptools brings us several useful features (see the main setup script
# docstring) but if it causes problems for our users or even only
# inconveniences them, they tend to complain about why we are using setuptools
# at all. Therefore we try to use setuptools as silently as possible, with a
# clear error message displayed to the user, but only in cases when we
# absolutely need setuptools.
#
# Setuptools usage logic:
# 1. attempt to use a preinstalled setuptools version
# 2. if a preinstalled setuptools version is not available, attempt to install
#    and use the most recent tested compatible setuptools release, possibly
#    downloading the installation from PyPI into the current folder
# 3. if we still do not have setuptools available, fall back to using distutils
#
# Note that we have made a slight trade-off here and chose to reuse an existing
# setuptools installation if available instead of always downloading and
# installing the most recent compatible setuptools release.
#
# Alternative designs and rationale for selecting the current design:
#
# distutils/setuptools usage:
#   * always use distutils
#       - misses out on all the setuptools features
#   * use setuptools if available, or fall back to using distutils
#       - chosen design
#       - gets us setuptools features if available, with clear end-user error
#         messages in case an operation is triggered that absolutely requires
#         setuptools to be available
#       - see below for notes on different setuptools installation alternatives
#   * always use setuptools
#       - see below for notes on different setuptools installation alternatives
#
# setuptools installation:
#   * expect setuptools to be preinstalled
#       - if not available, burden the user with installing setuptools manually
#       - see below for notes on using different setuptools versions
#   * use preinstalled setuptools if possible, or fall back to installing it
#     on-demand
#       - chosen design
#       - see below for notes on using different setuptools versions
#       - automated setuptools installations, and especially in-place upgrades,
#         can fail for various reasons (see below)
#       - reduces the risk of a stalled download stalling the whole setup
#         operation, e.g. because of an unavailable or unresponsive DNS server
#   * always install setuptools
#       - automated setuptools installations, and especially in-place upgrades,
#         can fail for various reasons (see below)
#       - user has no way to avoid setuptools installation issues by installing
#         setuptools himself, which would force us to make our setuptools
#         installation support universally applicable and that is just not
#         possible, e.g. users might be wanting to use customized package
#         management or their own package index instead of PyPI.
#
# setuptools version usage:
#   - basic problem here is that our project can be and has been tested only
#     with certain setuptools versions
#   - using a different version may have issues causing our setup procedures to
#     fail outside our control, either due to a setuptools bug or due to an
#     incompatibility between our implementation and the used setuptools
#     version
#   - some setuptools releases have known regressions, e.g. setuptools 3.0
#     which is documented to fail on Python 2.6 due to an accidental backward
#     incompatibility corrected in the setuptools 3.1 release
#   * allow using any setuptools version
#       - chosen design
#       - problems caused by incompatible setuptools version usage are
#         considered highly unlikely and can therefore be patched up as needed
#         when and if they appear
#       - users will most likely not have a setuptools version preinstalled
#         into their Python environment that is incompatible with that
#         environment
#       - if there is no setuptools version installed, our setup will attempt
#         to install the most recent tested setuptools release
#   * allow using only tested setuptools versions or possibly more recent ones
#       - unless we can automatically install a suitable setuptools version, we
#         will need to burden the user with installing it manually or fall back
#         to using distutils
#       - automated setuptools installations, and especially in-place upgrades,
#         can fail for various reasons (see below)
#       - extra implementation effort required compared to the chosen design,
#         with no significant benefit
#
# Some known scenarios causing automated setuptools installation failures:
#   * Download failures, e.g. because user has no access to PyPI.
#   * In-place setuptool upgrades can fail due to a known setuptools issue (see
#     'https://bitbucket.org/pypa/setuptools/issue/168') when both the original
#     and the new setuptools version is installed as a zipped egg distribution.
#     Last seen using setuptools 3.4.4.
#   * If the Python environment running our setup has already loaded setuptools
#     packages, then upgrading that installation in-place will fail with an
#     error message instructing the user to do the upgrade manually.
#   * When installing our project using pip, pip will load setuptools
#     internally, and it typically uses an older setuptools version which can
#     trigger the in-place upgrade failure as described above. What is worse,
#     we ignore this failure, we run into the following combination problem:
#       * pip calls our setup twice - once to collect the package requirement
#         information and once to perform the actual installation, and we do
#         not want to display multiple potentially complex error messages to
#         user for what is effectively the same error.
#       * Since we can not affect how external installers call our setup, to
#         avoid this we would need to either:
#           * Somehow cache the information that we already attempted and
#             failed to upgrade setuptools (complicated + possibly not robust).
#           * Patch the setuptools installation script to not display those
#             error messages (we would prefer to not be forced to maintain our
#             own patches for this script and use it as is).
#           * Avoid the issue by never upgrading an existing setuptools
#             installation (chosen design).

def acquire_setuptools_setup():
    if not attempt_to_use_setuptools:
        return

    def import_setuptools_setup():
        try:
            from setuptools import setup
        except ImportError:
            return
        return setup

    setup = import_setuptools_setup()
    if setup or not attempt_to_install_setuptools:
        return setup
    if (3,) <= sys.version_info[:2] < (3, 2):
        # Setuptools contains a test module with an explicitly specified UTF-8
        # BOM, which is not supported by Python's py2to3 tool prior to Python
        # 3.2 (see Python issue #7313). Setuptools can still be installed
        # manually using its ez_setup.py installer script (it will report and
        # ignore the error), but if we use the ez_setup.use_setuptools()
        # programmatic setup invocation from here - it will fail.
        #
        # There are several things that could be done here - patches welcome if
        # anyone actually needs them:
        #  - the issue could be worked around by running the setuptools
        #    installation as a separate process in this case
        #  - warning display could be more cleanly integrated into distutils
        #    command execution process so the warning does not get displayed
        #    if setuptools would not actually be useful, e.g. if user just ran
        #    our setup script with no command or with the --help option
        print("---")
        print("WARNING: can not install setuptools automatically using Python "
            "3.0 & 3.1")
        print("WARNING: if needed, install setuptools manually before running "
            "this installation using Python prior to version 3.2")
        print("---")
        return
    import suds_devel.ez_setup_versioned
    ez_setup = suds_devel.ez_setup_versioned.import_module()
    try:
        # Since we know there is no setuptools package in the current
        # environment, this will:
        # 1. download a setuptools source distribution to the current folder
        # 2. prepare an installable setuptools egg distribution in the current
        #    folder
        # 3. schedule for the prepared setuptools distribution to be installed
        #    together with our package (if our package is getting installed at
        #    all and setup has not been called for some other purpose, e.g.
        #    displaying its help information or running a non-install related
        #    setup command)
        ez_setup.use_setuptools()
    except (KeyboardInterrupt, SystemExit):
        raise
    except Exception:
        return
    return import_setuptools_setup()

setup = acquire_setuptools_setup()
using_setuptools = bool(setup)
if not using_setuptools:
    # Fall back to using distutils.
    from distutils.core import setup


# -----------------------------------------------------------------------------
# Support functions.
# -----------------------------------------------------------------------------

def read_python_code(filename):
    "Returns the given Python source file's compiled content."
    file = open(filename, "rt")
    try:
        source = file.read()
    finally:
        file.close()
    # Python 2.6 and below did not support passing strings to exec() &
    # compile() functions containing line separators other than '\n'. To
    # support them we need to manually make sure such line endings get
    # converted even on platforms where this is not handled by native text file
    # read operations.
    source = source.replace("\r\n", "\n").replace("\r", "\n")
    return compile(source, filename, "exec")

def recursive_package_list(*packages):
    """
    Returns a list of all the given packages and all their subpackages.

    Given packages are expected to be found relative to this script's location.

    Subpackages are detected by scanning the given packages' subfolder
    hierarchy for any folders containing the '__init__.py' module. As a
    consequence, namespace packages are not supported.

    This is our own specialized setuptools.find_packages() replacement so we
    can avoid the setuptools dependency.

    """
    result = set()
    todo = []
    for package in packages:
        folder = os.path.join(script_folder, *package.split("."))
        if not os.path.isdir(folder):
            raise Exception("Folder not found for package '%s'." % (package,))
        todo.append((package, folder))
    while todo:
        package, folder = todo.pop()
        if package in result:
            continue
        result.add(package)
        for subitem in os.listdir(folder):
            subpackage = ".".join((package, subitem))
            subfolder = os.path.join(folder, subitem)
            if not os.path.isfile(os.path.join(subfolder, "__init__.py")):
                continue
            todo.append((subpackage, subfolder))
    return list(result)

# Shamelessly stolen from setuptools project's pkg_resources module.
def safe_version(version_string):
    """
    Convert an arbitrary string to a standard version string

    Spaces become dots, and all other non-alphanumeric characters become
    dashes, with runs of multiple dashes condensed to a single dash.

    """
    version_string = version_string.replace(" ", ".")
    return re.sub("[^A-Za-z0-9.]+", "-", version_string)

def unicode2ascii(unicode):
    """Convert a unicode string to its approximate ASCII equivalent."""
    return unicode.encode("ascii", 'xmlcharrefreplace').decode("ascii")


# -----------------------------------------------------------------------------
# Load the suds library version information.
# -----------------------------------------------------------------------------

# Load the suds library version information directly into this module without
# having to import the whole suds library itself. Importing the suds package
# would have caused problems like the following:
#   * Forcing the imported package module to be Python 3 compatible without any
#     lib2to3 fixers first being run on it (since such fixers get run only
#     later as a part of the setup procedure).
#   * Making the setup module depend on the package module's dependencies, thus
#     forcing the user to install them manually (since the setup procedure that
#     is supposed to install them automatically will not be able to run unless
#     they are already installed).
# We execute explicitly compiled source code instead of having the exec()
# function compile it to get a better error messages. If we used exec() on the
# source code directly, the source file would have been listed as just
# '<string>'.
exec(read_python_code(os.path.join(script_folder, "suds", "version.py")))


# -----------------------------------------------------------------------------
# Custom setup.py 'test' command for running the project test suite.
# -----------------------------------------------------------------------------
# pytest and any of its requirements not already installed in the target Python
# environment will be automatically downloaded from PyPI and installed into the
# current folder as zipped egg distributions.
#
# Requirements:
#   - setup must be using setuptools
#   - if running Python version prior to 2.5, a suitable pytest version must
#     already be installed and will not be installed on demand (see the related
#     embedded code comment below for more detailed information)
#
# If the requirements are not met, the command simply reports an end-user error
# message explaining why the test functionality is unavailable.
#
# Since Python's distutils framework does not allow passing all received
# command-line arguments to its commands, it does not seem easy to customize
# how pytest runs its tests this way. To have better control over this, user
# should run the pytest on the target source tree directly, possibly after
# first building a temporary one to work around problems like Python 2/3
# compatibility.

def test_requirements():
    """
    Return test requirements for the 'test' command or an error string.

    An error is reported if the requirements can not be satisfied for some
    reason.

    Exact required packages and their versions vary depending on our target
    Python environment version as pytest dropped backward compatibility support
    for some of the Python versions we still support in this project.

    """
    if not using_setuptools:
        return "test command not available without setuptools"

    include_pytest_requirements = True

    if sys.version_info < (2, 5):
        # pytest requirements can not be installed automatically by this setup
        # script under Python 2.4.x environment. Specific pytest & py library
        # package version combination that we found working in Python 2.4.x
        # environments does not formally satisfy pytest requirements, and we
        # found no way to make setuptools' test command succeed when this
        # script installs packages that do not have all their formal
        # requirements satisfied.
        have_pytest, have_py = check_Python24_pytest_requirements()
        if not have_pytest:
            return "compatible preinstalled pytest needed prior to Python 2.5"
        if not have_py:
            return "compatible preinstalled py needed prior to Python 2.5"

        # We must not explicitly specify pytest requirements when running the
        # tests using a Python 2.4.x environment as the only way we found we
        # can run our tests there is to use formally incompatible pytest & py
        # packages. Explicitly specifying pytest requirements here would then
        # cause setuptols to verify those requirements prior to running our
        # test suite.
        include_pytest_requirements = False

    if ((3,) <= sys.version_info < (3, 2, 3)):
        # Python 3.x versions prior to Python 3.2.3 have a bug in their inspect
        # module causing inspect.getmodule() calls to fail if some module lazy
        # loads other modules when some of its attributes are accessed. For
        # more detailed information see Python development issue #13487
        # (http://bugs.python.org/issue13487).
        #
        # This occurs when using setuptools to install our project into a
        # Python 3.1 environment. There the py.error module seems to do such
        # lazy loading. Forcing that module to be loaded here, before the
        # setuptools installation procedure, avoids the issue.
        try:
            import py.error
            py.error.__attribute_access_to_force_this_module_to_lazy_load__
        except (AttributeError, ImportError):
            pass

    # When using Python 2.5 on Windows, if setuptools chooses to install the
    # colorama package (pytest requirement on Windows) older than version
    # 0.1.11, running our 'setup.py test' command may show benign error
    # messages caused by some colorama atexit handlers getting triggered
    # multiple times. There are no adverse effects to this and the issue only
    # occurs if the package is not already installed and it is the 'setup.py
    # test' command that is installing it. The issue has been fixed by the next
    # Python 2.5 compatible colorama 0.3.2 release.
    result = []
    if include_pytest_requirements:
        result.extend(pytest_requirements())
    result.extend(six_requirements())
    return result

test_error = None
tests_require = test_requirements()
if isinstance(tests_require, str):
    test_error = tests_require
else:
    extra_setup_params["tests_require"] = tests_require

if test_error:
    import distutils.cmd
    import distutils.errors

    class TestCommand(distutils.cmd.Command):
        description = test_error
        user_options = []
        def initialize_options(self):
            pass
        def finalize_options(self):
            pass
        def run(self):
            raise distutils.errors.DistutilsPlatformError(self.description)
else:
    from setuptools.command.test import (normalize_path as _normalize_path,
        test as _test)

    class TestCommand(_test):
        # Derived from setuptools.command.test.test for its
        # with_project_on_sys_path() method.

        # The test build can not be done in-place with Python 3+ as it requires
        # py2to3 conversion which we do not want modifying our original project
        # sources.
        if sys.version_info < (3,):
            description = "run pytest based unit tests after an in-place build"
        else:
            description = "run pytest based unit tests after a build"

        # Override base class's command-line options.
        #TODO: pytest argument passing support could be improved if we could
        # get distutils/setuptools to pass all unrecognized command-line
        # parameters to us instead of complaining about them.
        user_options = [("pytest-args=", "a", "arguments to pass to pytest "
            "(whitespace separated, whitespaces in arguments not supported)")]

        def initialize_options(self):
            self.pytest_args = None

        def finalize_options(self):
            self.test_args = []
            if self.pytest_args is not None:
                self.test_args = self.pytest_args.split()

        def run(self):
            # shamelessly lifted from setuptools.command.test.test.run()
            dist = self.distribution
            if dist.install_requires:
                dist.fetch_build_eggs(dist.install_requires)
            if dist.tests_require:
                dist.fetch_build_eggs(dist.tests_require)

            cmd = self._test_cmd_string()
            if self.dry_run:
                self.announce("skipping '%s' (dry run)" % (cmd,))
            else:
                self.announce("running '%s'" % (cmd,))
                self.with_project_on_sys_path(self.run_tests)

        def run_tests(self):
            import pytest
            sys.exit(pytest.main(self.test_args))

        def _test_cmd_string(self):
            parts = ["pytest"]
            if self.pytest_args:
                parts.append(self.pytest_args)
            return " ".join(parts)

distutils_cmdclass["test"] = TestCommand


# -----------------------------------------------------------------------------
# Mark the original suds project as obsolete.
# -----------------------------------------------------------------------------

if sys.version_info >= (2, 5):
    # distutils.setup() 'obsoletes' parameter not introduced until Python 2.5.
    extra_setup_params["obsoletes"] = ["suds"]


# -----------------------------------------------------------------------------
# Avoid setup warnings when constructing a list of all project sources.
# -----------------------------------------------------------------------------
# Part of this workaround implemented and part in the project's MANIFEST.in
# template. See a related comment in MANIFEST.in for more detailed information.

dummy_tools_folder = os.path.join(tools_folder, "__dummy__")
dummy_tools_file = os.path.join(dummy_tools_folder, "readme.txt")
try:
    if not os.path.isdir(dummy_tools_folder):
        os.mkdir(dummy_tools_folder)
    if not os.path.isfile(dummy_tools_file):
        f = open(dummy_tools_file, "w")
        try:
            f.write("""\
Dummy empty folder added as a part of a patch to silence setup.py warnings when
determining which files belong to the project. See a related comment in the
project's MANIFEST.in template for more detailed information.

Both the folder and this file have been generated by the project's setup.py
script and should not be placed under version control.
""")
        finally:
            f.close()
except EnvironmentError:
    # Something went wrong attempting to construct the dummy file. Ah well, we
    # gave it our best. Continue on with possible spurious warnings.
    pass


# -----------------------------------------------------------------------------
# Set up project metadata and run the actual setup.
# -----------------------------------------------------------------------------
# Package meta-data needs may be specified as:
#  * Python 2.x - UTF-8 encoded bytes
#  * Python [2.6, 3.0> - unicode string
#      - unicode strings containing non-ASCII characters supported since Python
#        commit 4c683ec4415b3c4bfbc7fe7a836b949cb7beea03
#  * Python [3.0, 3.2.2>
#      - may only contain ASCII characters due to a distutils bug (given input
#        can only be a unicode string and is encoded using the user's default
#        system code-page, e.g. typically CP1250 on eastern European Windows,
#        CP1252 on western European Windows, UTF-8 on Linux or any other)
#      - setuptools 3.5 works around the issue by overriding relevant distutils
#        functionality, allowing the use of non-ASCII characters, but only for
#        Python 3.1
#  * Python 3.2.2+ - unicode string
#      - unicode strings containing non-ASCII characters supported since Python
#        commit fb4d2e6d393e96baac13c4efc216e361bf12c293

can_not_use_non_ASCII_meta_data = (3,) <= sys.version_info < (3, 2, 2)
if (can_not_use_non_ASCII_meta_data and using_setuptools and
        sys.version_info[:2] == (3, 1)):
    from setuptools import __version__ as setuptools_version
    from pkg_resources import parse_version as pv
    can_not_use_non_ASCII_meta_data = pv(setuptools_version) < pv("3.5")

# Wrap long_description at 72 characters since the PKG-INFO package
# distribution metadata file stores this text with an 8 space indentation.
long_description = """
---------------------------------------
Lightweight SOAP client (Community fork).
---------------------------------------

  Based on the original 'suds' project by Jeff Ortel (jortel at redhat
dot com) hosted at 'http://fedorahosted.org/suds'.

  'Suds' is a lightweight SOAP-based web service client for Python
licensed under LGPL (see the LICENSE.txt file included in the
distribution).
"""

package_name = "suds-community"
version_tag = safe_version(__version__)
project_url = "https://github.com/suds-community/suds"
base_download_url = project_url + "/archive"
download_distribution_name = "release-%s.tar.gz" % (version_tag)
download_url = "%s/%s" % (base_download_url, download_distribution_name)

maintainer="Jurko Gospodnetić"
if can_not_use_non_ASCII_meta_data:
    maintainer = unicode2ascii(maintainer)

setup(
    name=package_name,
    version=__version__,
    description="Lightweight SOAP client (community fork)",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=["SOAP", "web", "service", "client"],
    url=project_url,
    download_url=download_url,
    packages=recursive_package_list("suds"),

    author="Jeff Ortel",
    author_email="jortel@redhat.com",
    maintainer=maintainer,
    maintainer_email="jurko.gospodnetic@pke.hr",

    # See PEP-301 for the classifier specification. For a complete list of
    # available classifiers see
    # 'http://pypi.python.org/pypi?%3Aaction=list_classifiers'.
    classifiers=["Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: "
            "GNU Library or Lesser General Public License (LGPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet"],

    # PEP-314 states that, if possible, license & platform should be specified
    # using 'classifiers'.
    license="(specified using classifiers)",
    platforms=["(specified using classifiers)"],

    # Register distutils command customizations.
    cmdclass=distutils_cmdclass,
    python_requires=">=3.5",

    **extra_setup_params)



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Zip compression related utilities.

"""

import os
import os.path
import sys
import zipfile

from suds_devel.utility import path_iter


def zip_folder_content(folder, zip_file):
    success = False
    zippy = zipfile.ZipFile(zip_file, "w", _zip_compression())
    try:
        archiver = _Archiver(zippy, folder)
        for root, folders, files in os.walk(folder):
            archiver.add_folder_with_files(root, files)
        success = True
    finally:
        zippy.close()
        if not success:
            os.remove(zip_file)


class _Archiver:

    def __init__(self, zip_file, folder):
        self.__zip_file = zip_file
        self.__base_folder_parts = list(path_iter(folder))

    def add_folder_with_files(self, folder, files):
        path_prefix = self.__path_prefix(folder)
        for file in files:
            assert file
            file_path = os.path.join(folder, file)
            self.__zip_file.write(file_path, path_prefix + file)
        # If no files are present in this folder and this is not the base
        # folder then we need to add the folder itself as an explicit entry.
        if not files and path_prefix:
            # Old Python versions did not support using the ZipFile.write()
            # method on folders so we do it manually by adding a 0-size entry
            # using ZipFile.writestr(). Encountered using Python 2.4.3.
            # N.B. An archived folder name must include a trailing slash, which
            # is exactly what we have in our prepared path_prefix value.
            self.__zip_file.writestr(path_prefix, "")

    def __path_prefix(self, folder):
        """
        Path prefix to be used when archiving any items from the given folder.

        Expects the folder to be located under the base folder path and the
        returned path prefix does not include the base folder information. This
        makes sure we include just the base folder's content in the archive,
        and not the base folder itself.

        """
        path_parts = path_iter(folder)
        _skip_expected(path_parts, self.__base_folder_parts)
        result = "/".join(path_parts)
        if result:
            result += "/"
        return result


# Mimic the next() built-in introduced in Python 2.6. Does not need to be
# perfect but only as good as we need it internally in this module.
if sys.version_info < (2, 6):
    def _iter_next(iter):
        return iter.next()
else:
    _iter_next = next


def _skip_expected(iter, expected):
    for expected_value in expected:
        value = _iter_next(iter)
        assert value == expected_value


_zip_compression_value = None

def _zip_compression():
    global _zip_compression_value
    if _zip_compression_value is None:
        try:
            import zlib
            _zip_compression_value = zipfile.ZIP_DEFLATED
        except ImportError:
            _zip_compression_value = zipfile.ZIP_STORED
    return _zip_compression_value



# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jeff Ortel ( jortel@redhat.com )

import sys
sys.path.append('../../')

from suds import *
from suds.client import Client

from datetime import datetime


errors = 0
url = 'http://localhost:8080/axis2/services/BasicService?wsdl'
print('url=%s' % (url,))

#
# create a service client using the wsdl.
#
client = Client(url)

#
# print the service (introspection)
#
print(client)

print('printList()')
print(client.service.printList(['a', 'b']))

#
# create a name object using the wsdl
#
print('create name')
name = client.factory.create('ns2:Name')
name.first = u'jeff'+unichr(1234)
name.last = 'ortel'

print(name)

#
# create a phone object using the wsdl
#
print('create phone')
phoneA = client.factory.create('ns2:Phone')
phoneA.npa = 410
phoneA.nxx = 822
phoneA.number = 5138

phoneB = client.factory.create('ns2:Phone')
phoneB.npa = 919
phoneB.nxx = 606
phoneB.number = 4406

#
# create a dog
#
dog = client.factory.create('ns2:Dog')
print(dog)
dog.name = 'Chance'
dog.trained = True
print(dog)

#
# create a person object using the wsdl
#
person = client.factory.create('ns2:Person')

#
# inspect empty person
#
print('{empty} person=\n%s' % (person,))

person.name = name
person.age = None
person.birthday = datetime.now()
person.phone.append(phoneA)
person.phone.append(phoneB)
person.pets.append(dog)

#
# inspect person
#
print('person=\n%s' % (person,))

#
# add the person (using the webservice)
#
print('addPersion()')
result = client.service.addPerson(person)
print('\nreply(\n%s\n)\n' % (result.encode('utf-8'),))

#
# create a new name object used to update the person
#
newname = client.factory.create('ns2:Name')
newname.first = 'Todd'
newname.last = None

#
# update the person's name (using the webservice) and print return person object
#
print('updatePersion()')
result = client.service.updatePerson(person, newname)
print('\nreply(\n%s\n)\n' % (str(result),))
result = client.service.updatePerson(person, None)
print('\nreply(\n%s\n)\n' % (str(result),))


#
# invoke the echo service
#
print('echo()')
client.service.echo(None)
result = client.service.echo('this is cool')
print('\nreply( %s )\n' % (str(result),))

print('echo() with {none}')
result = client.service.echo(None)
print('\nreply( %s )\n' % (str(result),))

#
# invoke the hello service
#
print('hello()')
result = client.service.hello()
print('\nreply( %s )\n' % (str(result),))

#
# invoke the testVoid service
#
try:
    print('getVoid()')
    result = client.service.getVoid()
    print('\nreply( %s )\n' % (str(result),))
except (KeyboardInterrupt, SystemExit):
    raise
except Exception:
    print(sys.exc_info()[1])

#
# test list args
#
print('getList(list)')
mylist = ['my', 'dog', 'likes', 'steak']
result = client.service.printList(mylist)
print('\nreply( %s )\n' % (str(result),))
# tuple
print('testListArgs(tuple)')
mylist = ('my', 'dog', 'likes', 'steak')
result = client.service.printList(mylist)
print('\nreply( %s )\n' % (str(result),))

#
# test list returned
#
for n in range(0, 3):
    print('getList(str, %d)' % (n,))
    result = client.service.getList('hello', n)
    print('\nreply( %s )\n' % (str(result),))
    assert ( isinstance(result, list) and len(result) == n )

print('addPet()')
dog = client.factory.create('ns2:Dog')
dog.name = 'Chance'
dog.trained = True
print(dog)
try:
    result = client.service.addPet(person, dog)
    print('\nreply( %s )\n' % (str(result),))
except (KeyboardInterrupt, SystemExit):
    raise
except Exception:
    print(sys.exc_info()[1])

print('___________________ E X C E P T I O N S __________________________')

#
# test exceptions
#
try:
    print('throwException() faults=True')
    result = client.service.throwException()
    print('\nreply( %s )\n' % (tostr(result),))
except (KeyboardInterrupt, SystemExit):
    raise
except Exception:
    print(sys.exc_info()[1])

#
# test faults
#
try:
    print('throwException() faults=False')
    client.set_options(faults=False)
    result = client.service.throwException()
    print('\nreply( %s )\n' % (tostr(result),))
except (KeyboardInterrupt, SystemExit):
    raise
except Exception:
    print(sys.exc_info()[1])

print('\nfinished: errors=%d' % (errors,))



# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jeff Ortel ( jortel@redhat.com )

"""
Provides XSD typing classes.

"""

from suds.sax import Namespace
from suds.sax.text import Text
from suds.sudsobject import Object


class Typer:
    """
    Provides XML node typing as either automatic or manual.

    @cvar types: Class to XSD type mapping.
    @type types: dict

    """

    types = {
        bool: ("boolean", Namespace.xsdns),
        float: ("float", Namespace.xsdns),
        int: ("int", Namespace.xsdns),
        int: ("long", Namespace.xsdns),
        str: ("string", Namespace.xsdns),
        Text: ("string", Namespace.xsdns),
        str: ("string", Namespace.xsdns)}

    @classmethod
    def auto(cls, node, value=None):
        """
        Automatically set the node's xsi:type attribute based on either
        I{value}'s or the node text's class. When I{value} is an unmapped
        class, the default type (xs:any) is set.

        @param node: XML node.
        @type node: L{sax.element.Element}
        @param value: Object that is or would be the node's text.
        @type value: I{any}
        @return: Specified node.
        @rtype: L{sax.element.Element}

        """
        if value is None:
            value = node.getText()
        if isinstance(value, Object):
            known = cls.known(value)
            if known.name is None:
                return node
            tm = known.name, known.namespace()
        else:
            tm = cls.types.get(value.__class__, cls.types.get(str))
        cls.manual(node, *tm)
        return node

    @classmethod
    def manual(cls, node, tval, ns=None):
        """
        Set the node's xsi:type attribute based on either I{value}'s or the
        node text's class. Then adds the referenced prefix(s) to the node's
        prefix mapping.

        @param node: XML node.
        @type node: L{sax.element.Element}
        @param tval: XSD schema type name.
        @type tval: str
        @param ns: I{tval} XML namespace.
        @type ns: (prefix, URI)
        @return: Specified node.
        @rtype: L{sax.element.Element}

        """
        xta = ":".join((Namespace.xsins[0], "type"))
        node.addPrefix(Namespace.xsins[0], Namespace.xsins[1])
        if ns is None:
            node.set(xta, tval)
        else:
            ns = cls.genprefix(node, ns)
            qname = ":".join((ns[0], tval))
            node.set(xta, qname)
            node.addPrefix(ns[0], ns[1])
        return node

    @classmethod
    def genprefix(cls, node, ns):
        """
        Generate a prefix.

        @param node: XML node on which the prefix will be used.
        @type node: L{sax.element.Element}
        @param ns: Namespace needing a unique prefix.
        @type ns: (prefix, URI)
        @return: I{ns} with a new prefix.
        @rtype: (prefix, URI)

        """
        for i in range(1, 1024):
            prefix = "ns%d" % (i,)
            uri = node.resolvePrefix(prefix, default=None)
            if uri in (None, ns[1]):
                return prefix, ns[1]
        raise Exception("auto prefix, exhausted")

    @classmethod
    def known(cls, object):
        try:
            md = object.__metadata__
            known = md.sxtype
            return known
        except Exception:
            pass



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Assertion test utility functions shared between multiple test modules.

Extracted into a separate module, named so pytest would apply its assertion
rewriting to it.

'pytest' assertion rewriting allows our assertion test utility functions to use
Python assertions and have them work even when run with Python assertions
disabled.

"""


def assert_no_output(pytest_capture_fixture):
    """
    Test utility asserting there was no captured stdout or stderr output.

    pytest_capture_fixture parameter may be one of pytest's output capture
    fixtures, e.g. capsys or capfd.

    """
    out, err = pytest_capture_fixture.readouterr()
    assert not out
    assert not err



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Generic exception classes shared in different development utility modules.

"""

class EnvironmentSetupError(Exception):
    """
    Signal to move onto setting up the next environment.

    Raised after the current environment's setup fails. Error message stored in
    the exception is expected to be reported to the user.

    """
    def __init__(self, message):
        Exception.__init__(self, message)



# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jeff Ortel ( jortel@redhat.com )

"""
Basic HTTP transport implementation classes.

"""

from suds.properties import Unskin
from suds.transport import *

import base64
from http.cookiejar import CookieJar
import http.client
import socket
import sys
import urllib.request, urllib.error, urllib.parse
import gzip
import zlib

from logging import getLogger
log = getLogger(__name__)


class HttpTransport(Transport):
    """
    Basic HTTP transport implemented using using urllib2, that provides for
    cookies & proxies but no authentication.

    """

    def __init__(self, **kwargs):
        """
        @param kwargs: Keyword arguments.
            - B{proxy} - An HTTP proxy to be specified on requests.
                 The proxy is defined as {protocol:proxy,}
                    - type: I{dict}
                    - default: {}
            - B{timeout} - Set the URL open timeout (seconds).
                    - type: I{float}
                    - default: 90

        """
        Transport.__init__(self)
        Unskin(self.options).update(kwargs)
        self.cookiejar = CookieJar()
        self.proxy = {}
        self.urlopener = None

    def open(self, request):
        try:
            url = self.__get_request_url_for_urllib(request)
            headers = request.headers
            log.debug('opening (%s)', url)
            u2request = urllib.request.Request(url, headers=headers)
            self.proxy = self.options.proxy
            return self.u2open(u2request)
        except urllib.error.HTTPError as e:
            raise TransportError(str(e), e.code, e.fp)

    def send(self, request):
        url = self.__get_request_url_for_urllib(request)
        msg = request.message
        headers = request.headers
        if 'Content-Encoding' in headers:
            encoding = headers['Content-Encoding']
            if encoding == 'gzip':
                msg = gzip.compress(msg)
            elif encoding == 'deflate':
                msg = zlib.compress(msg)
        try:
            u2request = urllib.request.Request(url, msg, headers)
            self.addcookies(u2request)
            self.proxy = self.options.proxy
            request.headers.update(u2request.headers)
            log.debug('sending:\n%s', request)
            fp = self.u2open(u2request, timeout=request.timeout)
            self.getcookies(fp, u2request)
            headers = fp.headers
            if sys.version_info < (3, 0):
                headers = headers.dict
            message = fp.read()
            if 'Content-Encoding' in headers:
                encoding = headers['Content-Encoding']
                if encoding == 'gzip':
                    message = gzip.decompress(message)
                elif encoding == 'deflate':
                    message = zlib.decompress(message)
            reply = Reply(http.client.OK, headers, message)
            log.debug('received:\n%s', reply)
            return reply
        except urllib.error.HTTPError as e:
            if e.code not in (http.client.ACCEPTED, http.client.NO_CONTENT):
                raise TransportError(e.msg, e.code, e.fp)

    def addcookies(self, u2request):
        """
        Add cookies in the cookiejar to the request.

        @param u2request: A urllib2 request.
        @rtype: u2request: urllib2.Request.

        """
        self.cookiejar.add_cookie_header(u2request)

    def getcookies(self, fp, u2request):
        """
        Add cookies in the request to the cookiejar.

        @param u2request: A urllib2 request.
        @rtype: u2request: urllib2.Request.

        """
        self.cookiejar.extract_cookies(fp, u2request)

    def u2open(self, u2request, timeout=None):
        """
        Open a connection.

        @param u2request: A urllib2 request.
        @type u2request: urllib2.Request.
        @return: The opened file-like urllib2 object.
        @rtype: fp

        """
        tm = timeout or self.options.timeout
        url = self.u2opener()
        return url.open(u2request, timeout=tm)

    def u2opener(self):
        """
        Create a urllib opener.

        @return: An opener.
        @rtype: I{OpenerDirector}

        """
        if self.urlopener is None:
            return urllib.request.build_opener(*self.u2handlers())
        return self.urlopener

    def u2handlers(self):
        """
        Get a collection of urllib handlers.

        @return: A list of handlers to be installed in the opener.
        @rtype: [Handler,...]

        """
        return [urllib.request.ProxyHandler(self.proxy)]

    def u2ver(self):
        """
        Get the major/minor version of the urllib2 lib.

        @return: The urllib2 version.
        @rtype: float

        """
        try:
            return float('%d.%d' % sys.version_info[:2])
        except Exception as e:
            return 0

    def __deepcopy__(self, memo={}):
        clone = self.__class__()
        p = Unskin(self.options)
        cp = Unskin(clone.options)
        cp.update(p)
        return clone

    @staticmethod
    def __get_request_url_for_urllib(request):
        """
        Returns the given request's URL, properly encoded for use with urllib.

        We expect that the given request object already verified that the URL
        contains ASCII characters only and stored it as a native str value.

        urllib accepts URL information as a native str value and may break
        unexpectedly if given URL information in another format.

        Python 3.x httplib.client implementation must be given a unicode string
        and not a bytes object and the given string is internally converted to
        a bytes object using an explicitly specified ASCII encoding.

        Python 2.7 httplib implementation expects the URL passed to it to not
        be a unicode string. If it is, then passing it to the underlying
        httplib Request object will cause that object to forcefully convert all
        of its data to unicode, assuming that data contains ASCII data only and
        raising a UnicodeDecodeError exception if it does not (caused by simple
        unicode + string concatenation).

        Python 2.4 httplib implementation does not really care about this as it
        does not use the internal optimization present in the Python 2.7
        implementation causing all the requested data to be converted to
        unicode.

        """
        assert isinstance(request.url, str)
        return request.url


class HttpAuthenticated(HttpTransport):
    """
    Provides basic HTTP authentication for servers that do not follow the
    specified challenge/response model. Appends the I{Authorization} HTTP
    header with base64 encoded credentials on every HTTP request.

    """

    def open(self, request):
        self.addcredentials(request)
        return HttpTransport.open(self, request)

    def send(self, request):
        self.addcredentials(request)
        return HttpTransport.send(self, request)

    def addcredentials(self, request):
        credentials = self.credentials()
        if None not in credentials:
            credentials = ':'.join(credentials)
            if sys.version_info < (3, 0):
                encodedString = base64.b64encode(credentials)
            else:
                encodedBytes = base64.urlsafe_b64encode(credentials.encode())
                encodedString = encodedBytes.decode()
            request.headers['Authorization'] = 'Basic %s' % encodedString

    def credentials(self):
        return self.options.username, self.options.password



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify it under
# the terms of the (LGPL) GNU Lesser General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library Lesser General Public License
# for more details at ( http://www.gnu.org/licenses/lgpl.html ).
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# written by: Jurko Gospodnetić ( jurko.gospodnetic@pke.hr )

"""
Basic configuration support shared in different development utility scripts.

"""

import os.path
import sys

# Must not use the six Python 2/3 compatibility package from here as this
# module gets used from the script for setting up basic development
# environments, and that script needs to be runnable even before the six
# package has been installed.
if sys.version_info < (3,):
    import ConfigParser as configparser
else:
    import configparser

from suds_devel.environment import Environment
import suds_devel.utility as utility


class BadConfiguration(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)


class Config(object):

    # Typed option values.
    BOOLEAN_TRUE  = ('1', 'yes', 'true', 'on', '+')
    BOOLEAN_FALSE = ('0', 'no', 'false', 'off', '-')
    IF_NEEDED     = ('maybe', '?',
                     'ifneeded', 'if needed',
                     'asneeded', 'as needed',
                     'ondemand', 'on demand')

    class TriBool:
        Yes = object()
        IfNeeded = object()
        No = object()

    def __init__(self, script, project_folder, ini_file):
        """
        Initialize new script configuration.

        External configuration parameters may be specified relative to the
        following folders:
          * script - relative to the current working folder
          * project_folder - relative to the script folder
          * ini_file - relative to the project folder

        """
        self.__init_script_folder(script)
        self.__init_project_folder(project_folder)
        self.__init_ini_file(ini_file)
        self.__init_reader()

    def _get_bool(self, section, option):
        x = self._reader.get(section, option).lower()
        if x in self.BOOLEAN_TRUE:
            return True
        if x in self.BOOLEAN_FALSE:
            return False
        raise BadConfiguration("Option '%s.%s' must be a boolean value." % (
            section, option))

    def _get_tribool(self, section, option):
        x = self._reader.get(section, option).lower()
        if x in self.BOOLEAN_TRUE:
            return Config.TriBool.Yes
        if x in self.BOOLEAN_FALSE:
            return Config.TriBool.No
        if x in self.IF_NEEDED:
            return Config.TriBool.IfNeeded
        raise BadConfiguration("Option '%s.%s' must be Yes, No or IfNeeded." %
            (section, option))

    def _read_environment_configuration(self):
        section_prefix = "env:"
        command_option = "command"
        self.python_environments = []
        for section in self._reader.sections():
            if section.lower().startswith(section_prefix):
                name = section[len(section_prefix):]
                command = self._reader.get(section, command_option)
                if not command:
                    raise BadConfiguration("'%s.%s' environment command "
                        "configuration option must not be empty." % (section,
                        command_option))
                self.python_environments.append(Environment(name, command))

    def __init_ini_file(self, ini_file):
        self.ini_file = os.path.join(self.project_folder, ini_file)
        self.ini_file = os.path.normpath(self.ini_file)
        if not os.path.isfile(self.ini_file):
            raise BadConfiguration("Missing configuration file '%s'." % (
                self.ini_file,))

    def __init_project_folder(self, project_folder):
        p = os.path.normpath(os.path.join(self.script_folder, project_folder))
        if not os.path.isdir(p):
            raise BadConfiguration("Could not find project folder '%s'." % p)
        self.project_folder = p

    def __init_reader(self):
        try:
            f = open(self.ini_file, "r")
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            raise BadConfiguration("Can not access configuration file '%s' - "
                "%s." % (self.ini_file, sys.exc_info()[1]))
        try:
            print("Reading configuration file '%s'..." % (self.ini_file,))
            self._reader = configparser.ConfigParser()
            self._reader.readfp(f)
        finally:
            f.close()

    def __init_script_folder(self, script):
        self.script_folder = utility.script_folder(script)
        if not self.script_folder:
            raise BadConfiguration("Could not determine script folder.")
